/**
 * Writer for http response.
 * @memberof module:pon/lib/responding
 * @constructor PonRes
 * @param {object} options - Optional settings
 * @param {string} [options.jsonSpaces=2] - Spaces for json response.
 * @param {string} [options.jsonContentType='application/json; charset=UTF-8;'] - Content type header for json response.
 *
 */

"use strict";

var extend = require('extend');

/** @lends PonRes */
function PonRes(config) {
    var s = this;
    extend(s, config || {});
}

PonRes.prototype = {
    jsonSpaces: 2,
    jsonContentType: 'application/json; charset=UTF-8;',
    /**
     * Send json to response.
     * @param {ServerResponse} res - Repose to write.
     * @param {object} data - Data to write.
     */
    sendJson: function (res, data) {
        var s = this;
        if (typeof(data) === 'string') {
            data = JSON.parse(data);
        }
        var body = JSON.stringify(data, null, s.jsonSpaces);
        s.setHeaders(res, {
            'Content-Length': Buffer.byteLength(body),
            'Content-Type': s.jsonContentType
        });
        s.flush(res, body);
        return s;
    },
    /**
     * Set header data.
     * @param {ServerResponse} res - Response to write.
     * @param {object} data - Header data.
     */
    setHeaders: function (res, data) {
        var s = this;
        Object.keys(data).forEach(function (key) {
            res.setHeader(key, data[key]);
        });
        return s;
    },
    /**
     * Send data data.
     * @param {ServerResponse} res - Response to write.
     * @param {object} data - Data to send.
     * @param {object} [options] - Optional settings.
     * @param {string} [options.format='json'] - Format of response data.
     * @param {object} [options.headers] - Headers to add.
     */
    send: function (res, data, options) {
        var s = this;
        options = options || {};
        var format = String(options.format || 'json').trim().toLowerCase(),
            headers = options.headers || {};
        s.setHeaders(res, headers);
        switch (format) {
            case 'json':
                s.sendJson(res, data);
                break;
            default:
                throw new Error('Unknown format: ' + format);
        }
    },
    /**
     * Flush response.
     * @param {ServerResponse} res - Response to flush.
     * @param {object} data - Data to flush.
     */
    flush: function (res, data) {
        var s = this;
        res.end(data);
        return s;
    }
};

module.exports = PonRes;

/**
 * @external ServerResponse
 */
